@@ -1,7 +1,11 @@
 from webob import Request, Response
 from webob.exc import HTTPFound
 from genshi.template import TemplateLoader
-import inspect, pickle, base64, types
+import inspect, pickle, base64, types, urllib
+from paste.registry import StackedObjectProxy
+
+request = StackedObjectProxy(name="request")
+response = StackedObjectProxy(name="response")
 
 def expose(template = None,
            content_type = 'text/html'):
@@ -20,6 +24,35 @@ def expose(template = None,
         return f
     return decorate
 
+def url(base_url, params={}):
+    if base_url.startswith('/'):
+        base_url = request.environ['SCRIPT_NAME'] + base_url
+
+    if params:
+        sane_params = {}
+        for key in params.iterkeys():
+            sane_params[key] = params[key].encode('utf-8')
+        return '?'.join((base_url, urllib.urlencode(sane_params)))
+
+    return base_url
+
+
+def redirect(where):
+    exc = HTTPFound(location=where)
+    try:
+        exc.identity = response.identity
+    except:
+        pass
+
+    if response.flash_obj:
+        decoded_flash = pickle.dumps(response.flash_obj)
+        exc.set_cookie('flash_obj', base64.b64encode(decoded_flash))
+
+    raise exc
+
+def flash(msg, css_class='warning'):
+    response.flash_obj = {'msg':msg, 'class':css_class}
+
 class Controller(object):
     def __init__(self, application, template_path, helpers):
         self.template_path = template_path
@@ -42,22 +75,9 @@ class Controller(object):
         stream = tmpl.generate(**params)
         return stream.render('xhtml')
 
-    def redirect(self, response, where):
-        exc = HTTPFound(location=where)
-        try:
-            exc.identity = response.identity
-        except:
-            pass
-
-        if response.flash_obj:
-            decoded_flash = pickle.dumps(response.flash_obj)
-            exc.set_cookie('flash_obj', base64.b64encode(decoded_flash))
-
-        raise exc
-
     @expose()
-    def not_found(self, request, response, args, params):
-        response.headers['Content-Type'] = 'text/html'
+    def not_found(self, args, params):
+        response.content_type = 'text/html'
         response.status = 404
         return '''
 <html>
@@ -103,32 +123,28 @@ class Controller(object):
             else:
                 call = call.index
 
-        response = self.do_call(call, request, path[:])
-        return response
+        self.do_call(call, request, path[:])
 
     def do_call(self, call, request, path):
-        response = Response(body="Artichoke Default Page", status=200)
-        response.headerlist=[('Content-type', call.artichoke['content-type'])]
+        response.content_type = call.artichoke['content-type']
         self.inject_tools(request, response)
 
         if call.artichoke['template']:
             tmpl_context = {}
+            tmpl_context['a'] = type('Bunch', (object,), {'url':staticmethod(url)})
             tmpl_context['h'] = self.helpers
             tmpl_context['request'] = request
             tmpl_context['response'] = response
-            tmpl_context.update(call(request, response, path, request.params))
+            tmpl_context.update(call(path, request.params))
 
             template = call.artichoke['template'] + '.html'
             response.body = self.render(template, tmpl_context)
         else:
-            response.body = call(request, response, path, request.params)
+            response.body = call(path, request.params)
 
         return response
 
     def inject_tools(self, request, response):
-        def flash(self, msg, css_class='warning'):
-            self.flash_obj = {'msg':msg, 'class':css_class}
-
         response.flash_obj = {}
         if request.cookies.get('flash_obj'):
             try:
@@ -137,5 +153,3 @@ class Controller(object):
             except Exception, e:
                 print e
             response.delete_cookie('flash_obj')
-
-        response.flash = types.MethodType(flash, response, Response)
\ No newline at end of file
